//=============================================================================
// InfernoMappack.InfernoMap.
// *** for use as an Embedded Mutator in DM-A4_DawnsInferno ONLY ***
// Removes the Player's Enforcer - weapon choice is configurable.
// Sets Timelimit to no timelimit.
// Sets FragLimit to no fraglimit.
// Sets TransLocator to not kill ScriptedPawns.
// Controls a trigger when the WarLord is killed.
// Ends the game when WarLord Killer passes through Pearly Gates.
// An Embedded Mutator for placing INSIDE a special UT Map.
// Author:  Dawn Knight
// Tested only on version 4.36
// Version A4.4  11/17/2003 04:12:11 PM
//=============================================================================
// To embed this Mutator in a Map:
// 1. Make sure this Mutator is NOT listed in the "EditPackages" section of your
//    UnrealTournament.ini file!  This is VERY Important!
// 2. Make sure "InfernoMap.u" IS located in your UnrealTournament/System
//    Folder.
// 3. Start a fresh UED session and open your Map for editing.
// 4. In the Command entry field at the bottom of the UED window type:
//
//         obj load file=InfernoMap.u package=MyLevel
//
// 5. Open the Actor
//
//         Actor->Info->Mutator->InfernoMap
//
// 6. Highlight "InfernoMap" in the browser and then select a surface
//    somewhere in your map, right-click on the surface, and in the pop-up
//    menu that appears, choose "Add InfernoMap Here".  Place only ONE
//    instance of this Mutator in your Map.
// 7. The resulting Actor that is placed in your map can be double-clicked on
//    to bring up the "InfernoMap Properties" browser where you can select
//    the Enforcer Replacement weapon.
// 8. Build and Save your map - the embedded Mutator code with your settings
//    will be saved away in the map.
//
// If you subsequently delete the single instance of the Mutator from your map,
// and then reBuild and reSave, the Mutator code will be deleted from the map
// and you will have to repeat all these steps to embed it again.
//=============================================================================
class InfernoMap expands Mutator; //config (InfernoMap);

var bool bPreBPInitialized;
var string TheReplacement;  // Weapon that will replace Enforcer
var DeathMatchPlus DM;
enum ERWeaponType // Replacement Weapon type.
{
  No_Weapon,
  Chainsaw,
  Enforcer,
  DoubleEnforcer,
  ImpactHammer,
  Minigun2,
  PulseGun,
  Ripper,
  ShockRifle,
  SuperShockRifle,
  SniperRifle,
  UT_BioRifle,
  UT_Eightball,
  UT_FlakCannon,
  Redeemer
};

var Pawn Conquerer; // The Player that kills the WarLord
var Mover InnerSanctumGate;  // The Gate to Inner Sanctum
var Trigger ThroneTrigger; // The Trigger that opens Gate behind Throne

// Variables which the Level designer can adjust:
var() int BotMaxFallDamage; // the max damage a Bot can incur in a fall
var() ERWeaponType EnforcerReplacement; // accessible to Level Designer

// PreBeginPlay will be used mainly to Bootstrap load this Mutator by linking it
// into the MutatorList as the 1st Mutator in a possible chain of Mutators.
function PreBeginPlay()
{
  local bool LoggedAlready;
  local Mutator M, Previous;
  local Mover Mvr;
  local Trigger T;
  local ZoneInfo ZI;
  local WarLord WL;

  if ( !bPreBPInitialized ) // older versions of UT call this function twice but we
                            // only want to run our code once!
  {
    bPreBPInitialized = True;
    DM = DeathMatchPlus(Level.Game);

    // Add the mutator by linking it into the MutatorList ala Beppo.
    // this Embedded Mutator makes sense to be 1st after the BaseMutator
    // in the Mutatorlist...
    // But first, check for and destroy an already loaded version
    for (M = Level.Game.BaseMutator; M != None; M = M.NextMutator)
    {
      if (GetItemName(string(M.class)) == GetItemName(string(Self.class))) //Already loaded version - Destroy it!
      {
        Previous.NextMutator = M.NextMutator;
        if (M != Self) // This is the other version
        {
          UnLinkDamageMutator(M); // do this BEFORE M.Destroy()!
          M.Destroy();
          LoggedAlready = True;
        }
      }
      else  // this will happen at least the 1st time thru the For Loop.
      {
        Previous = M; // So Previous can now be used as the Previous Mutator
                      // in the next pass thru the for Loop.
      }
    }
    if (!LoggedAlready)
    {
      Log("");
      Log("**** InfernoMap A4.3: Embedded Mutator for DM-A4_DawnsInferno");
      Log("");
    }

    // We need to Bootstrap this instance into the List
    Self.NextMutator = Level.Game.BaseMutator.NextMutator; // Make a place in the List
    Level.Game.BaseMutator.NextMutator = Self; // place it 1st after BaseMutator
    Level.Game.RegisterDamageMutator(self);
    TheReplacement = GetRWeaponType(); //What's the Enforcer replacement weapon?

    // Set game variables
		if (DM != none)
		{
			Level.Game.bNoMonsters = false;
			Level.Game.bHumansOnly = false;
			DM.FragLimit = 0;
			DM.TimeLimit = 0;
			DM.RemainingTime = 0;
		}

		// Find the InnerSanctum Gate
    foreach AllActors( class 'Mover', Mvr, 'GateLevel3' )
    {
    	InnerSanctumGate = Mvr;
    }

    // Find the Throne Trigger
    foreach AllActors( class 'Trigger', T, 'WarLordDeath' )
    {
    	ThroneTrigger = T;
    }

		// Find the 'AscendToHeaven' Zones and force their ZoneGravity.Z values
    foreach AllActors( class 'ZoneInfo', ZI, 'AscendToHeaven' )
    {
    	// Some Mutators may insist on altering ZoneGravity and we can't allow
    	// that in the special 'AscendToHeaven' Zones, so we override it here.
			ZI.ZoneGravity.Z = 200.0;
    }
  }
}

function UnLinkDamageMutator(Mutator M)
{
  local int i, count;
  local Mutator C, Previous;

  for (C = Level.Game.DamageMutator; C != none; C = C.NextDamageMutator)
  {
    if (C == M)
    {
      count = i;
      break;
    }
    i++;
    Previous = C;
  }
  if (count == 0)
  {
    Level.Game.DamageMutator = Level.Game.DamageMutator.NextDamageMutator;
  }
  else
  {
    Previous.NextDamageMutator = C.NextDamageMutator;
  }
}

//==============================================================================
// We already added ourselves to the Mutator List in our PreBeginPlay, but
// GameInfo.InitGame() will erroneously add us again unless we intercept the
// call with our own AddMutator function and prevent it.
function AddMutator(Mutator M)
{
  if (M == Self)
  {
    return; // Don't add us.
  }
  super.AddMutator(M);  // keep the chain unbroken
}

function string GetRWeaponType()
{
  switch( EnforcerReplacement )  // accessible to Level Designer
  {
    case No_Weapon:
      return "Botpack.None";
    case Chainsaw:
      return "Botpack.Chainsaw";
    case Enforcer:
      return "Botpack.Enforcer";
    case DoubleEnforcer:
      return "Botpack.DoubleEnforcer";
    case ImpactHammer:
      return "Botpack.ImpactHammer";
    case Minigun2:
      return "Botpack.Minigun2";
    case PulseGun:
      return "Botpack.PulseGun";
    case Ripper:
      return "Botpack.Ripper";
    case ShockRifle:
      return "Botpack.ShockRifle";
    case SuperShockRifle:
      return "Botpack.SuperShockRifle";
    case SniperRifle:
      return "Botpack.SniperRifle";
    case UT_BioRifle:
      return "Botpack.UT_BioRifle";
    case UT_Eightball:
      return "Botpack.UT_Eightball";
    case UT_FlakCannon:
      return "Botpack.UT_FlakCannon";
    case Redeemer:
      return "Botpack.WarheadLauncher";
    default:
      return "Botpack.Chainsaw";
  }
}

// Called just before PlayerPawn is Spawned.  We use it to adjust a Player's
// Inventory to our liking
function ModifyPlayer( Pawn PlayerPawn )
{
  local Inventory TheWeapon, Inv;
	local NavigationPoint startSpot;

  if (!PlayerPawn.IsA('Spectator'))
  {
    TheWeapon = PlayerPawn.FindInventoryType(class'Enforcer');
    if (TheWeapon != None)
    {
      TheWeapon.DropInventory(); // Remove it from Player's Inventroy
      TheWeapon.Destroy(); // Remove it from the game
    }

    TheWeapon = PlayerPawn.FindInventoryType(class'ImpactHammer');
    if (TheWeapon != None)
    {
      TheWeapon.DropInventory(); // Remove it from Player's Inventroy
      TheWeapon.Destroy(); // Remove it from the game
    }

    if (TheReplacement != "Botpack.None")
    {
      DM.GiveWeapon(PlayerPawn,TheReplacement);  // Give Replacement for Enforcer
    }
    // Once Inner Sanctum Gate is open, it goes to state 'Mover'.  Use this to
    // switch to new PlayerStarts for Bots-only inside the Inner Sanctum.
    if (PlayerPawn.IsA('Bot') && (InnerSanctumGate.IsInState('Mover')))
		{
			startSpot = InnerSanctumBotStart(PlayerPawn); // Guaranteed or we wouldn't be here
			PlayerPawn.SetLocation(startSpot.Location);
			startSpot.PlayTeleportEffect(PlayerPawn, true);
			PlayerPawn.SetRotation(startSpot.Rotation);
			PlayerPawn.ViewRotation = PlayerPawn.Rotation;
			PlayerPawn.ClientSetLocation( startSpot.Location, startSpot.Rotation );

			DM.GiveWeapon(PlayerPawn,"Botpack.ImpactHammer");
			DM.GiveWeapon(PlayerPawn,"Botpack.PulseGun");
			DM.GiveWeapon(PlayerPawn,"Botpack.ShockRifle");
			DM.GiveWeapon(PlayerPawn,"Botpack.UT_Eightball");

			for( Inv=PlayerPawn.Inventory; Inv!=None; Inv=Inv.Inventory )
			{
				if (Ammo(Inv)!=None)
				{
					Ammo(Inv).AmmoAmount  = 999;
					Ammo(Inv).MaxAmmo  = 999;
				}
			}
		}
	}

  if ( NextMutator != None )
    NextMutator.ModifyPlayer(PlayerPawn);
}

function NavigationPoint InnerSanctumBotStart(Pawn Player)
{
	local PlayerStart Dest, Candidate[16], Best;
	local float Score[16], BestScore, NextDist;
	local pawn OtherPlayer;
	local int i, num;
	local Teleporter Tel;
	local NavigationPoint N;

	// choose candidates
	for ( N=Level.NavigationPointList; N!=None; N=N.NextNavigationPoint )
	{
		Dest = PlayerStart(N);
		// InnerSanctum Bot Starts are just unenabled PlayerStarts
		if ( (Dest != None) && !Dest.bEnabled && !Dest.Region.Zone.bWaterZone )
		{
			if (num<16)
				Candidate[num] = Dest;
			else if (Rand(num) < 16)
				Candidate[Rand(16)] = Dest;
			num++;
		}
	}
	if (num>16) num = 16;

	// assess candidates
	for (i=0;i<num;i++)
	{
		if (Candidate[i].TeamNumber == 63) // 63 means Throne Room PlayerStart
			Score[i] =  (1500 * FRand() ) + 1500; // Weight Throne Room PlayerStarts higher
		else
			Score[i] = 1500 * FRand(); //randomize
	}
	for ( OtherPlayer=Level.PawnList; OtherPlayer!=None; OtherPlayer=OtherPlayer.NextPawn)
		if ( OtherPlayer.bIsPlayer && (OtherPlayer.Health > 0) && !OtherPlayer.IsA('Spectator') )
			for ( i=0; i<num; i++ )
			{
				if ( OtherPlayer.Region.Zone == Candidate[i].Region.Zone )
				{
					Score[i] -= 1500;
					NextDist = VSize(OtherPlayer.Location - Candidate[i].Location);
					if ( NextDist < OtherPlayer.CollisionRadius + OtherPlayer.CollisionHeight )
						Score[i] -= 1000000.0;
					else if ( (NextDist < 2000) && FastTrace(Candidate[i].Location, OtherPlayer.Location) )
						Score[i] -= (10000.0 - NextDist);
				}
				else if ( DM.NumPlayers + DM.NumBots == 2 )
				{
					Score[i] += 2 * VSize(OtherPlayer.Location - Candidate[i].Location);
					if ( FastTrace(Candidate[i].Location, OtherPlayer.Location) )
						Score[i] -= 10000;
				}
			}

	BestScore = Score[0];
	Best = Candidate[0];
	for (i=1;i<num;i++)
		if (Score[i] > BestScore)
		{
			BestScore = Score[i];
			Best = Candidate[i];
		}
	return Best;
}

/******************************************************************************/
// To prevent Bots from too much Falling Damage
function MutatorTakeDamage( out int ActualDamage, Pawn Victim, Pawn InstigatedBy,
	out Vector HitLocation, out Vector Momentum, name DamageType)
{
	if ( Victim != none && InstigatedBy != none )
	{
	  if ( Victim.IsA('Bot') && DamageType == 'Fell' && ActualDamage > BotMaxFallDamage
			&& !InnerSanctumGate.IsInState('Mover') ) // Gate hasn't opened yet
	  {
	    ActualDamage = BotMaxFallDamage;
	  }
	  else
	  if ( Victim.IsA('SkaarjLord') && InstigatedBy.IsA('Bot') )
	  {
	  	ActualDamage = 0;
	  }
	  else
	  if ( Victim.IsA('WarLord') )
	  {
	  	if ( InstigatedBy.IsA('PlayerPawn') )
	  	{
	  		if ( DamageType == 'Jolted' )
	  		{
	  			ActualDamage /= 6;
	  		}
	  		else
	  		if ( DamageType == 'RedeemerDeath' )
	  		{
	  			ActualDamage /= 4;
	  		}
	//			Log("**** WarLord shot by"@InstigatedBy$", using"@DamageType$
	//				", Damage ="@ActualDamage$", at"@Level.TimeSeconds);
	  	}
	  	else
	  	{
	  		ActualDamage = 0;
			}
	  }
  }
// Deliberately do NOT pass MutatorTakeDamage to next Mutator(s)!
//	if ( NextDamageMutator != None )
//		NextDamageMutator.MutatorTakeDamage( ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType );
}

/******************************************************************************/
// To prevent TeleFragging Scripted Pawns
function bool PreventDeath(Pawn Killed, Pawn Killer, name damageType, vector HitLocation)
{
  if (Killer != none && Killed.IsA('ScriptedPawn') && damageType == 'gibbed')
  {
    return true;
  }

// Deliberately do NOT pass PreventDeath to next Mutator(s)!
//	if ( NextMutator != None )
//		return NextMutator.PreventDeath(Killed,Killer, damageType,HitLocation);
	return false;
}

/******************************************************************************/
// To Activate Throne Trigger when the WarLord is killed and force gibs when
// he dies.  Without gibs, a dead WarLord looks very unrealistic.
function ScoreKill(Pawn Killer, Pawn Other)
{
	if ( Other.IsA('WarLord') ) // Need EXACTLY ONE and ONLY ONE WarLord in Map!
	{
		Other.SpawnGibbedCarcass();
		Other.Destroy();
		Conquerer = Killer;
		ThroneTrigger.bInitiallyActive = true;
		DM.TimeLimit = 310;
		DM.RemainingTime = 310;
		SetTimer(1.0, true);
		BroadcastLocalizedMessage( class'InfernoMessage', 6, Conquerer.PlayerReplicationInfo );
	}
// Deliberately do NOT pass ScoreKill to next Mutator(s)!
//	if ( NextMutator != None )
//		NextMutator.ScoreKill(Killer, Other);
}

function Timer()
{
	switch (DM.RemainingTime)
	{
		case 305:
			BroadcastLocalizedMessage( class'InfernoMessage', 6, Conquerer.PlayerReplicationInfo );
			break;
		case 295:
		case 285:
		case 275:
		case 235:
		case 225:
		case 215:
		case 175:
		case 165:
		case 155:
		case 115:
		case 105:
		case 95:
		case 55:
		case 45:
		case 35:
		case 20:
		case 15:
			BroadcastLocalizedMessage( class'InfernoMessage', 7, Conquerer.PlayerReplicationInfo );
			break;
	}
}

defaultproperties
{
     DrawScale=5.0
     EnforcerReplacement="Botpack.None"
     BotMaxFallDamage=50
}

